@@ -21,23 +21,25 @@ import re
 import sys
 from base64 import b64decode
 from collections import namedtuple
-from typing import List, Union
+from typing import List, Union, Tuple
 
 from math import ceil, floor
 
 from PyQt5.QtCore import (Qt, QSize, pyqtSignal)
-from PyQt5.QtGui import QPainter, QFont, QColor, QPixmap, QFontMetrics, QKeyEvent, QStatusTipEvent, QMouseEvent
-from PyQt5.QtWidgets import QWidget, QApplication, QMenu, QSizePolicy, QAction, QInputDialog
+from PyQt5.QtGui import QPainter, QFont, QColor, QPixmap, QFontMetrics, QKeyEvent, QStatusTipEvent, QMouseEvent, QPen
+from PyQt5.QtWidgets import QApplication, QMenu, QSizePolicy, QAction, QInputDialog, QMessageBox, \
+	QAbstractScrollArea
 
 from pre_workbench.bbuf_parsing import apply_grammar_on_bbuf
 from pre_workbench import configs, guihelper
 from pre_workbench.algo.range import Range
 from pre_workbench.configs import SettingsSection
-from pre_workbench.guihelper import setClipboardText, showWidgetDlg, getClipboardText, APP
+from pre_workbench.guihelper import setClipboardText, showWidgetDlg, getClipboardText, APP, filledColorIcon
 from pre_workbench.app import GlobalEvents
 from pre_workbench.controls.hexview_selheur import SelectionHelpers
 from pre_workbench.objects import ByteBuffer, parseHexFromClipboard, BidiByteBuffer, ByteBufferList
 from pre_workbench.rangetree import RangeTreeWidget
+from pre_workbench.structinfo.format_info import builtinTypes
 from pre_workbench.structinfo.parser import parse_definition
 from pre_workbench.util import PerfTimer
 
@@ -72,8 +74,9 @@ configs.registerOption(group, 'backgroundColor', 'backgroundColor', 'color', {},
 pattern_heading = re.compile("[#]{0,6}")
 
 HitTestResult = namedtuple('HitTestResult', ['buffer', 'offset', 'region'])
+SelectionHeuristicMatch = namedtuple('SelectionHeuristicMatch', ['buffer', 'start', 'end', 'description', 'color'])
 
-class HexView2(QWidget):
+class HexView2(QAbstractScrollArea):
 	onNewSubflowCategory = pyqtSignal(str, object)
 	parseResultsUpdated = pyqtSignal(list)
 	selectionChanged = pyqtSignal(object)
@@ -88,9 +91,9 @@ class HexView2(QWidget):
 		logging.debug("HexView - formatInfoContainer = %r", self.formatInfoContainer)
 		self.annotationSetDefaultName = annotationSetDefaultName
 		self.buffers = list()
-		self.firstLine = 0
-		self.scrollY = 0
-		self.partialLineScrollY = 0
+		#self.firstLine = 0
+		#self.scrollY = 0
+		#self.partialLineScrollY = 0
 		self.setFocusPolicy(Qt.StrongFocus)
 
 		self.backgroundPixmap = QPixmap()
@@ -102,9 +105,13 @@ class HexView2(QWidget):
 		self.selBuffer = 0
 		self.selStart = 0
 		self.selEnd = 0
+		self.clickGrammarUndefRef = None
+		self.clickGrammarFieldRef = None
+		self.selHeurMatches = list()
 		self.itemY = list()
 		self.lastHit = None
 		self.selecting = False
+		self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
 		self.setContextMenuPolicy(Qt.CustomContextMenu)
 		self.customContextMenuRequested.connect(self._onCustomContextMenuRequested)
 		if byteBuffer is None:
@@ -114,6 +121,22 @@ class HexView2(QWidget):
 		self.setMouseTracking(True)
 		self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
 
+	@property
+	def scrollY(self):
+		return self.verticalScrollBar().value()
+
+	@scrollY.setter
+	def scrollY(self, value):
+		self.verticalScrollBar().setValue(value)
+
+	@property
+	def firstLine(self):
+		return floor(self.scrollY / self.dyLine)
+
+	@property
+	def partialLineScrollY(self):
+		return self.scrollY - (self.dyLine * self.firstLine)
+
 	def _loadOptions(self, *dummy):
 		self.bytesPerLine = configs.getValue('HexView2.general.bytesPerLine')
 		self.addressFormat = configs.getValue('HexView2.address.Format')
@@ -170,11 +193,8 @@ class HexView2(QWidget):
 		hit = self._hitTest(point)
 		ctxMenu = QMenu("Context menu", self)
 		if hit is not None:
-			if hit.offset < self.selStart or hit.offset > self.selEnd or hit.buffer != self.selBuffer:
-				self.selStart = self.selEnd = hit.offset
-				self.selBuffer = hit.buffer
-				self.selecting = False
-				self.redrawSelection()
+			if hit.offset < self.selFirst() or hit.offset > self.selLast() or hit.buffer != self.selBuffer:
+				self.select(hit.offset, hit.offset, hit.buffer, False)
 			self._buildSelectionContextMenu(ctxMenu)
 		else:
 			self._buildGeneralContextMenu(ctxMenu)
@@ -185,9 +205,27 @@ class HexView2(QWidget):
 		ctx.addAction("Copy Selection C Array", lambda: self.copySelection((", ", "0x%02X")))
 		ctx.addAction("Copy Selection Hexdump\tCtrl-Shift-C", lambda: self.copySelection("hexdump"))
 		#ctx.addAction("Copy selected annotations", lambda: self.copySelection("hexdump"))
+		if len(self.selHeurMatches) > 0:
+			mnu = ctx.addMenu("Selection Heuristic Matches")
+			for match in self.selHeurMatches:
+				mnu.addAction(filledColorIcon(match.color, 16), "[%d:%d] %s" % (match.start, match.end-1, match.description),
+							  lambda match=match: self.select(match.start, match.end-1, match.buffer))
 		ctx.addSeparator()
 		#ctx.addAction("Selection %d-%d (%d bytes)"%(self.selStart,self.selEnd,self.selLength()))
 		#ctx.addAction("Selection 0x%X - 0x%X (0x%X bytes)"%(self.selStart,self.selEnd,self.selLength()))
+		if self.clickGrammarUndefRef:
+			for name, (length, fun) in builtinTypes.items():
+				if length == self.selLength() and fun:
+					ctx.addAction(name, lambda name=name: self.clickGrammarAdd(name))
+			ctx.addAction(f"BYTES[{self.selLength()}]", lambda: self.clickGrammarAdd(f"BYTES[{self.selLength()}]"))
+			ctx.addAction(f"STRING[{self.selLength()}]", lambda: self.clickGrammarAdd(f"STRING[{self.selLength()}](charset=\"utf8\")"))
+			ctx.addAction("struct { ... }", lambda: self.clickGrammarAdd("struct { _undef_1 BYTES[" + str(self.selLength()) + "] }"))
+			ctx.addAction("bits { ... }", lambda: self.clickGrammarAdd("bits { _undef_1 : " + str(self.selLength()*8) + " }"))
+			ctx.addSeparator()
+		if self.clickGrammarFieldRef:
+			ctx.addAction("Undefine field " + self.clickGrammarFieldRef[0].field_name, self.clickGrammarUndefine)
+			ctx.addSeparator()
+
 		try:
 			match = next(
 				self.buffers[self.selBuffer].matchRanges(start=self.selFirst(), end=self.selLast()+1, doesntHaveMetaKey='_sdef_ref'))
@@ -219,7 +257,6 @@ class HexView2(QWidget):
 		for container_id, container, macroName in APP().find_macros_by_input_types(["BYTE_ARRAY"]):
 			menu.addAction(macroName, lambda c=container, name=macroName: c.getMacro(name).execute(self.getRangeBytes(self.selRange())))
 
-
 	def _buildGeneralContextMenu(self, ctx):
 		ctx.addAction("Select all", lambda: self.selectAll())
 		ctx.addSeparator()
@@ -279,6 +316,78 @@ class HexView2(QWidget):
 				lst.add(buf)
 			macro.execute(lst)
 
+	def clickGrammarAdd(self, definitionStr):
+		new_field_name = QInputDialog.getText(self, "New Field", "Please enter a name for the new field of type "+definitionStr+":")[0]
+		if not new_field_name: return
+		bytes_range, struct_range = self.clickGrammarUndefRef
+		bytes = bytes_range.metadata['_sdef_ref']
+		struct = struct_range.metadata['_sdef_ref']
+		if any(True for i, el in enumerate(struct.fi.children) if el[0] == new_field_name):
+			QMessageBox.warning(self, "Error", "The struct already contains a field named \"" + new_field_name + "\"")
+			return
+
+		sel = self.selRange()
+		idx = next(i for i, el in enumerate(struct.fi.children) if el[0] == bytes_range.field_name)
+
+		# resize or remove current _undef_
+		offset_in_bytes = sel.start - bytes_range.start
+		if offset_in_bytes > 0:
+			from pre_workbench.structinfo.expr import deserialize_expr
+			bytes.updateParams(size=deserialize_expr(str(offset_in_bytes)))
+			idx += 1
+		else:
+			del struct.fi.children[idx]
+
+		# insert new field
+		new_def = parse_definition(definitionStr)
+		new_def_size = sel.length()
+		struct.fi.children.insert(idx, (new_field_name, new_def, ''))
+		idx += 1
+
+		# insert new _undef_ behind new field, if needed
+		remaining_size = bytes_range.length() - offset_in_bytes - new_def_size
+		if remaining_size > 0:
+			new_bytes_def = parse_definition(f'BYTES[{remaining_size}]')
+			struct.fi.children.insert(idx, ('_undef_XXX', new_bytes_def, ''))
+
+		# re-number all _undef_ fields
+		self._clickGrammarRenumber(struct_range)
+
+		self.formatInfoContainer.write_file(self.formatInfoContainer.file_name)
+
+	def clickGrammarUndefine(self):
+		field_range, struct_range = self.clickGrammarFieldRef
+		struct = struct_range.metadata['_sdef_ref']
+
+		start = end = next(i for i, el in enumerate(struct.fi.children) if el[0] == field_range.field_name)
+		undef_len = field_range.length()
+		if start > 0 and struct.fi.children[start - 1][0].startswith('_undef_'):
+			undef_len += struct.fi.children[start - 1][1].fi.size_expr.evaluate_dict({})
+			start -= 1
+		if end + 1 < len(struct.fi.children) and struct.fi.children[end + 1][0].startswith('_undef_'):
+			undef_len += struct.fi.children[end + 1][1].fi.size_expr.evaluate_dict({})
+			end += 1
+
+		logging.debug(f"clickGrammarUndefine: replacing {start} to {end} with BYTES[{undef_len}]")
+		struct.fi.children[start:end+1] = [('_undef_', parse_definition(f'BYTES[{undef_len}]'), '')]
+
+		# re-number all _undef_ fields
+		self._clickGrammarRenumber(struct_range)
+
+		self.formatInfoContainer.write_file(self.formatInfoContainer.file_name)
+
+
+
+	def _clickGrammarRenumber(self, struct_range):
+		struct = struct_range.metadata['_sdef_ref']
+		number = 1
+		for i in range(len(struct.fi.children)):
+			name, fi, comment = struct.fi.children[i]
+			if name.startswith('_undef_'):
+				struct.fi.children[i] = (f'_undef_{number}', fi, comment)
+				number += 1
+		
+
 	def setDefaultAnnotationSet(self, name):
 		self.annotationSetDefaultName = name
 		self.loadAnnotations(name)
@@ -380,6 +489,7 @@ class HexView2(QWidget):
 	def _newSubflowCategory(self, category, parse_context, **kv):
 		logging.debug("on_new_subflow_category: %r",category)
 		self.onNewSubflowCategory.emit(category, parse_context)
+		logging.debug("on_new_subflow_category done: %r",category)
 
 	def _fiTreeItemSelected(self, item, previous):
 		if item is None: return
@@ -401,10 +511,8 @@ class HexView2(QWidget):
 			deltaY = e.pixelDelta().y()
 		if deltaY == 0: return
 		self.scrollY = max(0, min((self.maxLine()-1)*self.dyLine, self.scrollY - deltaY))
-		self.firstLine = floor(self.scrollY / self.dyLine)
-		self.partialLineScrollY = self.scrollY - (self.dyLine * self.firstLine)
 		logging.debug("wheelEvent deltaY=%d scrollY=%d partial=%d",deltaY,self.scrollY,self.partialLineScrollY)
-		self.redraw()
+		#self.redraw()
 
 	def scrollIntoView(self, bufIndex:int, offset:int):
 		line = self.byteOffsetToLineNumber(bufIndex, offset)
@@ -414,9 +522,8 @@ class HexView2(QWidget):
 			self._setFirstLine(line - 5)  #TODO - ich weiß vorher nicht, wie viele zeilen auf den schirm passen
 
 	def _setFirstLine(self, line):
-		self.firstLine = max(0, min(self.maxLine()-1, line))
-		self.scrollY = self.firstLine * self.dyLine
-		self.redraw()
+		self.scrollY = max(0, min(self.maxLine()-1, line)) * self.dyLine
+		#self.redraw()
 
 	############## MOUSE EVENTS - SELECTION  ############################
 
@@ -469,7 +576,8 @@ class HexView2(QWidget):
 		return (start,end)
 
 	def _hitTest(self, point):
-		x, y = point.x(), point.y()
+		dx = -self.horizontalScrollBar().value()
+		x, y = point.x()-dx, point.y()
 		linePos = None
 		if (x >= self.xAscii):
 			pos = floor((x - self.xAscii) / self.dxAscii)
@@ -519,11 +627,30 @@ class HexView2(QWidget):
 		#self.fiTreeWidget.hilightFormatInfoTree(r)
 
 		with PerfTimer("selectionChanged event handlers"):
+			shortest = None
+			self.clickGrammarUndefRef = None
+			self.clickGrammarFieldRef = None
 			if self.selBuffer < len(self.buffers):
 				self.selectionChanged.emit(r)
 
-			QApplication.postEvent(self, QStatusTipEvent("Buffer #%d  Selection %d-%d (%d bytes)   0x%X - 0x%X (0x%X bytes)"%(
-				self.selBuffer, self.selStart,self.selEnd,self.selLength(),self.selStart,self.selEnd,self.selLength())))
+				def is_fi_type(range, typeName):
+					return hasattr(range.metadata.get('_sdef_ref'), 'fi_type') and range.metadata['_sdef_ref'].fi_type == typeName
+
+				ranges = list(self.buffers[self.selBuffer].matchRanges(containsRange=self.selRange(), hasMetaKey='_sdef_ref'))
+				if len(ranges) >= 2 and is_fi_type(ranges[1], 'StructFI'):
+					if (is_fi_type(ranges[0], 'FieldFI') and ranges[0].metadata['_sdef_ref'].fi.format_type == 'BYTES'
+							and ranges[0].field_name.startswith('_undef_')):
+						self.clickGrammarUndefRef = (ranges[0], ranges[1])
+						logging.debug("clickGrammar Match! / bytes: %r / struct: %r",ranges[0],ranges[1])
+					else:
+						self.clickGrammarFieldRef = (ranges[0], ranges[1])
+				try:
+					shortest = min(ranges, key=len)
+				except ValueError:
+					pass
+
+			QApplication.postEvent(self, QStatusTipEvent("Buffer #%d  Selection %d-%d (%d bytes)   0x%X - 0x%X (0x%X bytes)    %r   %s"%(
+				self.selBuffer, self.selStart,self.selEnd,self.selLength(),self.selStart,self.selEnd,self.selLength(),shortest,"ClickGrammar Available!" if self.clickGrammarUndefRef else "")))
 
 
 	def selectRange(self, rangeObj, scrollIntoView=False):
@@ -596,10 +723,10 @@ class HexView2(QWidget):
 			super().keyPressEvent(e)
 
 	def focusInEvent(self, event) -> None:
-		self.update()
+		self.viewport().update()
 
 	def focusOutEvent(self, event) -> None:
-		self.update()
+		self.viewport().update()
 
 	def zoomIn(self):
 		self.fontHex.setPointSize(self.fontHex.pointSize() + 1)
@@ -633,7 +760,8 @@ class HexView2(QWidget):
 			self.buffers = bbuf
 		else:
 			raise TypeError("Invalid type passed to HexView2.setBuffer: "+str(type(bbuf)))
-		self.firstLine = 0
+		self.scrollY = 0
+		self.verticalScrollBar().setMaximum(self.maxLine() * self.dyLine)
 		self.parseResultsUpdated.emit([buf.fi_tree for buf in self.buffers])
 		self.select(0, 0, 0)
 		self.redraw()
@@ -642,16 +770,22 @@ class HexView2(QWidget):
 
 	def resizeEvent(self, e):
 		self.redraw()
+		self.verticalScrollBar().setPageStep(self.height())
+		self.horizontalScrollBar().setPageStep(self.width())
+		self.horizontalScrollBar().setMaximum(max(0, self.xAscii + self.dxAscii * (self.bytesPerLine+3) - self.width()))
 		#self.fiTreeWidget.resize(self.width() - self.fiTreeWidget.pos().x()-10, self.height()-40)
 
 	def sizeHint(self):
 		return QSize(self.xAscii + self.dxAscii * self.bytesPerLine + 10, 256)
 
+	def scrollContentsBy(self, dx, dy):
+		self.redraw()
+
 	def redraw(self):
 		self.pixmapsInvalid = True
 		for buffer in self.buffers:
 			buffer.invalidateCaches()
-		self.update()
+		self.viewport().update()
 
 	def drawPixmaps(self):
 		if self.size().height() < 3 or self.size().width() < 3: return
@@ -671,7 +805,7 @@ class HexView2(QWidget):
 			qpTxt.end()
 
 	def redrawSelection(self):
-		self.update()
+		self.viewport().update()
 
 	def paintEvent(self, e):
 		with PerfTimer("paintEvent"):
@@ -683,7 +817,7 @@ class HexView2(QWidget):
 				self.pixmapsInvalid = False
 			try:
 				qp = QPainter()
-				qp.begin(self)
+				qp.begin(self.viewport())
 				qp.drawPixmap(0, 0, self.backgroundPixmap)
 				self._drawSelection(qp)
 				self._drawHover(qp)
@@ -709,21 +843,20 @@ class HexView2(QWidget):
 	
 	def _drawLine(self, qpTxt, qpBg, lineNumber, y):
 		TXT_DY = self.fontAscent + self.linePadding
+		dx = -self.horizontalScrollBar().value()
 
 		bufIdx, offset, _ = self.lineNumberToByteOffset(lineNumber)
 		buffer = self.buffers[bufIdx]
 		if offset == 0:
 			# draw buffer separator
-			qpBg.fillRect(2,int(y),100,1,QColor("red"))
+			qpBg.fillRect(dx+2,int(y),100,1,QColor("red"))
 			qpTxt.setFont(self.fontSection[0]); qpTxt.setPen(self.fsAscii)
-			qpTxt.drawText(self.xHex, int(y+self.fontSection[0].pointSize() * 1.7), repr(buffer.metadata))
+			qpTxt.drawText(dx+self.xHex, int(y+self.fontSection[0].pointSize() * 1.7), repr(buffer.metadata))
 			y += self.fontSection[0].pointSize() * 2
 
 		end = min(len(buffer), offset + self.bytesPerLine)
 		ii = 0
 		for i in range(offset, end):
-			theByte = buffer.getByte(i)
-
 			# if specified, print section header
 			#sectionAnnotations = buffer.getAnnotationValues(start=i, annotationProperty="section");
 			sectionAnnotations = buffer.ranges.getMetaValuesStartingAt(i, "section")
@@ -733,40 +866,42 @@ class HexView2(QWidget):
 				for row in sectionAnnotations:
 					bangs = len(pattern_heading.match(row).group())
 					qpTxt.setFont(self.fontSection[bangs])
-					qpTxt.drawText(self.xHex, int(y+self.fontSection[bangs].pointSize() * 1.7), row)
+					qpTxt.drawText(dx+self.xHex, int(y+self.fontSection[bangs].pointSize() * 1.7), row)
 					y += self.fontSection[bangs].pointSize() * 2
 				qpTxt.setFont(self.fontAddress)
 				qpTxt.setPen(QColor("#555555"))
-				if (ii != 0): qpTxt.drawText(self.xAddress, int(y+TXT_DY), self.addressFormat.format(i));
+				if (ii != 0): qpTxt.drawText(dx+self.xAddress, int(y+TXT_DY), self.addressFormat.format(i));
 
 			if (ii == 0):  #print address for first byte in line
 				qpTxt.setFont(self.fontAddress)
 				qpTxt.setPen(self.fsAddress)
-				qpTxt.drawText(self.xAddress, int(y+TXT_DY), self.addressFormat.format(offset))
+				qpTxt.drawText(dx+self.xAddress, int(y+TXT_DY), self.addressFormat.format(offset))
 
 			# if specified, draw background color from style attribute
 			bg = buffer.getStyle(i, "color", None)
 			fg = buffer.getStyle(i, "textcolor", None)
 			if (bg):
-				qpBg.fillRect(self.xHex + ii * self.dxHex + int(ii/self.hexSpaceAfter)*self.hexSpaceWidth + 2, int(y+1), self.dxHex, self.dyLine-2, QColor(bg))
-				qpBg.fillRect(self.xAscii + ii * self.dxAscii, int(y+1), self.dxAscii, self.dyLine-2, QColor(bg))
+				qpBg.fillRect(dx+self.xHex + ii * self.dxHex + int(ii/self.hexSpaceAfter)*self.hexSpaceWidth + 2, int(y+1), self.dxHex, self.dyLine-2, QColor(bg))
+				qpBg.fillRect(dx+self.xAscii + ii * self.dxAscii, int(y+1), self.dxAscii, self.dyLine-2, QColor(bg))
 
 			# store item's Y position and buffer pos
 			self.itemY.append((bufIdx, i, int(y)))
 
 			# print HEX and ASCII representation of this byte
+			theByte = buffer.getByte(i)
 			qpTxt.setFont(self.fontHex)
 			qpTxt.setPen( self.fsHex if not fg else QColor(fg))
-			qpTxt.drawText(self.xHex + ii * self.dxHex + int(ii/self.hexSpaceAfter)*self.hexSpaceWidth + 2, int(y+TXT_DY), self.hexFormat.format(theByte))
+			qpTxt.drawText(dx+self.xHex + ii * self.dxHex + int(ii/self.hexSpaceAfter)*self.hexSpaceWidth + 2, int(y+TXT_DY), self.hexFormat.format(theByte))
 			qpTxt.setFont(self.fontAscii)
 			qpTxt.setPen(self.fsAscii if not fg else QColor(fg))
 			asciichar = chr(theByte) if (theByte > 0x20 and theByte < 0x80) else "."
-			qpTxt.drawText(self.xAscii + ii * self.dxAscii, int(y+TXT_DY), asciichar)
+			qpTxt.drawText(dx+self.xAscii + ii * self.dxAscii, int(y+TXT_DY), asciichar)
 			ii += 1
 
 		return y + self.dyLine
 
 	def _drawSelection(self, qp):
+		self.selHeurMatches.clear()
 		if len(self.buffers) == 0 or len(self.itemY) == 0: return
 
 		selMin = min(self.selStart, self.selEnd)
@@ -792,6 +927,28 @@ class HexView2(QWidget):
 				qp.fillRect(xHex, y, self.dxHex, dy, self.fsHover)
 				qp.fillRect(xAscii, y, self.dxAscii, dy, self.fsHover)
 
+			if len(self.selHeurMatches) > 0:
+				startY = self.height()-10
+				for match in self.selHeurMatches:
+					if match.buffer == self.lastHit.buffer and match.start <= self.lastHit.offset and match.end > self.lastHit.offset:
+						qp.setPen(QColor(match.color))
+						qp.drawText(10, startY, "[%d:%d] %s" % (match.start, match.end-1, match.description))
+						startY -= 15
+
+	def highlightMatch(self, qp: QPainter, matchrange: Tuple[int, int, int], desc: str, color: QColor):
+		(bufIdx, start, end) = matchrange
+		self.selHeurMatches.append(SelectionHeuristicMatch(bufIdx, start, end, desc, color))
+		((firstBuf, firstOffset), (lastBuf, lastOffset)) = self.visibleRange()
+		if bufIdx == firstBuf and start < firstOffset: start = firstOffset
+		for i in range(start, end):
+			(xHex, xAscii, y, dy) = self.offsetToClientPos(bufIdx, i)
+			if dy is None: break
+			p = QPen(color)
+			p.setWidth(3)
+			qp.setPen(p)
+			qp.drawLine(xHex + 3, y + dy, xHex + self.dxHex - 3, y + dy)
+			qp.drawLine(xAscii + 1, y + dy, xAscii + self.dxAscii - 2, y + dy)
+
 	########### CALCULATION    #########################
 	def lineNumberToByteOffset(self, lineNumber:int) -> HitTestResult:
 		bufOffset = lineNumber * self.bytesPerLine
@@ -812,11 +969,12 @@ class HexView2(QWidget):
 		return self.firstLine + ceil(len(self.itemY)/self.bytesPerLine)
 
 	def offsetToClientPos(self, buffer:int, offset:int):
+		dx = -self.horizontalScrollBar().value()
 		column = offset % self.bytesPerLine
 		for bufIdx, bufOffset, itemY in self.itemY:
 			if bufIdx == buffer and bufOffset == offset:
-				return (self.xHex + column * self.dxHex + int(column / self.hexSpaceAfter) * self.hexSpaceWidth,
-						self.xAscii + self.dxAscii * column,
+				return (dx+self.xHex + column * self.dxHex + int(column / self.hexSpaceAfter) * self.hexSpaceWidth,
+						dx+self.xAscii + self.dxAscii * column,
 						itemY + floor(self.linePadding * 0.5),
 						ceil(self.charHeight * 1.2))
 		logging.warning("trying to paint outside viewport %r", offset)
