@@ -21,14 +21,15 @@ from base64 import b64encode, b64decode
 from typing import Optional, List, Tuple
 
 import yaml
+from PyQt5 import QtCore
 from PyQt5.QtCore import (Qt, pyqtSignal, QAbstractItemModel, QModelIndex, pyqtSlot)
 from PyQt5.QtWidgets import QTextEdit, QTabWidget, QWidget, QVBoxLayout, \
     QMenu, \
-    QAbstractItemView, QTableView
+    QAbstractItemView, QTableView, QAction, QInputDialog, QMessageBox
 
 import pre_workbench.app
 
-from pre_workbench.configs import SettingsField
+from pre_workbench.configs import SettingsField, getIcon
 from pre_workbench.guihelper import getMonospaceFont, setClipboardText, getClipboardText, APP
 from pre_workbench.structinfo.expr import Expression
 from pre_workbench.controls.genericwidgets import showSettingsDlg, showListSelectDialog
@@ -143,7 +144,7 @@ class PacketListModel(QAbstractItemModel):
 
         return None
 
-    def index(self, row, column, parent):
+    def index(self, row, column, parent = QModelIndex()):
         if not self.hasIndex(row, column, parent):
             return QModelIndex()
 
@@ -181,12 +182,13 @@ class PacketListWidget(QWidget):
 
     def __init__(self):
         super().__init__()
+        self.lastFindExpression = ""
         self.initUI()
 
     def showData(self, data: List[ByteBuffer]):
         dv = DynamicDataWidget()
         dv.setContents(data)
-        dv.setWindowTitle("Data view")
+        dv.setWindowTitle("Item Details")
         pre_workbench.app.MainWindow.showChild(dv, True)
 
     def initUI(self):
@@ -216,21 +218,62 @@ class PacketListWidget(QWidget):
         self.packetlist.selectionModel().currentChanged.connect(self.onPacketlistCurrentChanged)
         #tabs.addTab(self.packetlist, "Raw Frames")
         layout.addWidget(self.packetlist)
+        self.ctx_item_actions = [
+            QAction(getIcon("document-search-result.png"), "Item Details", triggered=lambda: self.showData(self.getSelectedBuffers()), enabled=False),
+            QAction(getIcon("flag.png"), "Mark/Unmark Selected Buffers", triggered=self._markUnmarkSelection, enabled=False),
+        ]
+        self.ctx_general_actions = [
+            QAction(getIcon("binocular-flag.png"), "Find By Expression", triggered=self._findByExpression),
+        ]
+        self.actions = self.ctx_item_actions + self.ctx_general_actions + [
+            QAction(getIcon("table-reset.png"), "Reset Header", triggered=lambda: self.packetlistmodel.autoCols()),
+            QAction(getIcon("table-insert-column-tag.png"), "Add Metadata Column", triggered=lambda: self._quickAddDialog("Quick Add Metadata Column", self._getQuickAddMetadataElements(), None)),
+            QAction(getIcon("table-insert-column-bookmark.png"), "Add Field Column", triggered=lambda: self._quickAddDialog("Quick Add Field Column", self._getQuickAddFieldElements(), None)),
+        ]
+
+    def _findByExpression(self):
+        expr_str = QInputDialog.getText(self, "Find By Expression", "Please enter expression. All rows for which the expression is true will be selected.", text=self.lastFindExpression)[0]
+        if not expr_str: return
+        self.lastFindExpression = expr_str
+        expr = Expression(expr_str=expr_str)
+        matches = 0
+        model = self.packetlist.selectionModel()
+        model.clearSelection()
+        for rowIndex, buf in enumerate(self.packetlistmodel.listObject.buffers):
+            match = bool(expr.evaluate_bbuf(buf))
+            if match:
+                matches += 1
+                if matches == 1:
+                    self.packetlist.setCurrentIndex(self.packetlistmodel.index(rowIndex, 0))
+                model.select(self.packetlistmodel.index(rowIndex, 0),
+                             QtCore.QItemSelectionModel.Select | QtCore.QItemSelectionModel.Rows)
+            self.packetlistmodel.headerDataChanged.emit(Qt.Vertical, rowIndex, rowIndex)
+        QMessageBox.information(self, "Find By Expression", f"Of {len(self.packetlistmodel.listObject.buffers)} buffers, {matches} matched the expression " + expr.serialize())
 
     def _rowHeaderClicked(self, rowIndex: int):
         self.packetlistmodel.markPacket(rowIndex)
 
+    def selectMarked(self):
+        model = self.packetlist.selectionModel()
+        model.clearSelection()
+        for rowIndex, buf in enumerate(self.packetlistmodel.listObject.buffers):
+            if buf.metadata.get('marked'):
+                model.select(self.packetlistmodel.index(rowIndex, 0), QtCore.QItemSelectionModel.Select | QtCore.QItemSelectionModel.Rows)
+
     def setContents(self, lstObj: Optional[ByteBufferList]):
         self.listObject = lstObj
-        logging.debug("PacketListWidget::setContents %r %d", lstObj, len(lstObj))
+        #logging.debug("PacketListWidget::setContents %r %d", lstObj, len(lstObj))
         self.setWindowTitle(str(lstObj))
         self.packetlistmodel.setList(self.listObject)
+        self.meta_updated.emit("actions", self.actions)
 
     def onPacketlistSelectionChanged(self, selected, deselected):
         buffers = list()
         for index in self.packetlist.selectionModel().selectedRows():
             buffers.append(self.listObject.buffers[index.row()])
         self.meta_updated.emit("zoom", buffers)
+        for action in self.ctx_item_actions:
+            action.setEnabled(len(buffers) > 0)
 
     def onPacketlistCurrentChanged(self, current, previous):
         pass
@@ -242,14 +285,20 @@ class PacketListWidget(QWidget):
         index = self.packetlist.indexAt(point)
         ctx = QMenu("Context menu", self.packetlist)
         if index.isValid():
-            ctx.addAction("Item Details", lambda: self.showData(self.getSelectedBuffers()))
-            ctx.addAction("Mark/Unmark Packet", lambda: [self.packetlistmodel.markPacket(index.row()) for index in self.packetlist.selectionModel().selectedRows()])
+            for action in self.ctx_item_actions:
+                ctx.addAction(action)
             self._buildRunMacroOnBufferSubmenu(ctx, "Run Macro On" + (" Selected Buffers" if len(self.packetlist.selectionModel().selectedRows()) > 1 else " Buffer"))
             ctx.addSeparator()
+        ctx.addAction("Select Marked Buffers", lambda: self.selectMarked())
         ctx.addAction("Select All", lambda: self.packetlist.selectAll())
+        for action in self.ctx_general_actions:
+            ctx.addAction(action)
 
         ctx.exec(self.packetlist.viewport().mapToGlobal(point))
 
+    def _markUnmarkSelection(self):
+        [self.packetlistmodel.markPacket(index.row()) for index in self.packetlist.selectionModel().selectedRows()]
+
     def _buildRunMacroOnBufferSubmenu(self, ctx, title):
         menu = ctx.addMenu(title)
         for container_id, container, macroName in APP().find_macros_by_input_types(["BYTE_BUFFER", "BYTE_BUFFER_LIST"]):
@@ -266,36 +315,43 @@ class PacketListWidget(QWidget):
                 lst.add(self.listObject.buffers[index.row()])
             macro.execute(lst)
 
+    def _quickAddDialog(self, title: str, elements: List[Tuple[str, str]], addIdx: int):
+        def on_ok(keys):
+            for key in keys:
+                self.packetlistmodel.addColumn(ColumnInfo(key, key), addIdx)
+        showListSelectDialog(elements, None, title, self, on_ok, multiselect=True)
+
     def _generateQuickAddMenu(self, ctx: QMenu, title: str, elements: List[Tuple[str, str]], addIdx: int):
         if len(elements) > 20:
-            def on_ok(keys):
-                for key in keys:
-                    self.packetlistmodel.addColumn(ColumnInfo(key, key), addIdx)
-            ctx.addAction(title + " ...", lambda: showListSelectDialog(elements, None, title, self, on_ok, multiselect=True))
+            ctx.addAction(title + " ...", lambda: self._quickAddDialog(title, elements, addIdx))
         else:
             quick = ctx.addMenu(title)
             for key, text in elements:
                 quick.addAction(text, lambda key=key, text=text: self.packetlistmodel.addColumn(ColumnInfo(key, text), addIdx))
 
+    def _getQuickAddMetadataElements(self):
+        return [("${\"" + key + "\"}", "$" + key) for key in
+                    sorted(self.listObject.getAllKeys(metadataKeys=True, fieldKeys=False))]
+
+    def _getQuickAddFieldElements(self):
+        return [("fields[\""+key+"\"]", key) for key in
+                    sorted(self.listObject.getAllKeys(metadataKeys=False, fieldKeys=True))]
+
     def onHeaderContextMenu(self, point):
         index = self.packetlist.horizontalHeader().logicalIndexAt(point)
 
         ctx = QMenu("Context menu", self.packetlist)
         if index > -1:
-            ctx.addAction("Header "+str(index))
+            ctx.addAction("Header "+str(index)).setDisabled(True)
             ctx.addAction("Edit", lambda: self.onEditColumn(index))
             ctx.addAction("Remove Column", lambda: self.packetlistmodel.removeColumn(index))
             ctx.addSeparator()
         addIdx = None if index == -1 else index
         ctx.addAction("Add Column ...", lambda: self.onAddColumn(addIdx))
 
-        elements = [("${\"" + key + "\"}", "$" + key) for key in
-                    sorted(self.listObject.getAllKeys(metadataKeys=True, fieldKeys=False))]
-        self._generateQuickAddMenu(ctx, "Quick Add Metadata Column", elements, addIdx)
+        self._generateQuickAddMenu(ctx, "Quick Add Metadata Column", self._getQuickAddMetadataElements(), addIdx)
 
-        elements = [("fields[\""+key+"\"]", key) for key in
-                    sorted(self.listObject.getAllKeys(metadataKeys=False, fieldKeys=True))]
-        self._generateQuickAddMenu(ctx, "Quick Add Field Column", elements, addIdx)
+        self._generateQuickAddMenu(ctx, "Quick Add Field Column", self._getQuickAddFieldElements(), addIdx)
 
         ctx.addSeparator()
         magic = "!!pre_workbench/packetListHeaderState\n"
@@ -390,8 +446,8 @@ class ByteBufferWidget(QWidget):
             pass
 
     def _onParseResultsUpdated(self, fi_trees):
-        self.tabWidget.clear()
-        self.tabWidget.addTab(self.textbox, "Raw buffer")
+        #TODO this is the wrong place - it happens after the newSubflowCategory calls and clears them again immediately   self.tabWidget.clear()
+        #self.tabWidget.addTab(self.textbox, "Raw buffer")
         self.meta_updated.emit("grammar", fi_trees)
 
     def _newSubflowCategory(self, category, parse_context):
