@@ -24,13 +24,15 @@ from PyQt5.QtCore import (Qt)
 from PyQt5.QtGui import QColor
 from PyQt5.QtWidgets import QMenu, QFileDialog, QTreeWidget, QTreeWidgetItem, \
 	QTreeWidgetItemIterator, QMessageBox, QAction
+from pre_workbench.consts import SYNTAX_REFERENCE_URL
 
 from pre_workbench import configs, guihelper
 from pre_workbench.algo.range import Range
 from pre_workbench.configs import SettingsField
 from pre_workbench.controls.genericwidgets import showSettingsDlg
 from pre_workbench.controls.scintillaedit import showScintillaDialog
-from pre_workbench.guihelper import getMonospaceFont, filledColorIcon, setClipboardText
+from pre_workbench.guihelper import getMonospaceFont, filledColorIcon, setClipboardText, navigateBrowser, \
+	isOptionPressed
 from pre_workbench.interactive_fic import InteractiveFormatInfoContainer
 from pre_workbench.structinfo.format_info import FormatInfo, StructFI, VariantStructFI, SwitchFI, RepeatStructFI, \
 	UnionFI, BitStructFI
@@ -106,11 +108,14 @@ class RangeTreeWidget(QTreeWidget):
 	def _onCustomContextMenuRequested(self, point):
 		ctx = QMenu("Context menu", self)
 		if item := self.itemAt(point):
-			range = item.data(0, RangeTreeWidget.RangeRole)
+			item_range = item.data(0, RangeTreeWidget.RangeRole)
 			source = item.data(0, RangeTreeWidget.SourceDescRole)
 
 			parentSource = item.parent().data(0, RangeTreeWidget.SourceDescRole) if item.parent() else None
 			if isinstance(source, FormatInfo):
+				ctx.addAction("Edit ...", lambda: self.editField(source))
+				if isOptionPressed():
+					ctx.addAction("Edit tree ...", lambda: self.editField2(source))
 				if isinstance(source.fi, (StructFI, UnionFI)):
 					ctx.addAction("Add field ...", lambda: self.addField(source, "StructField"))
 					ctx.addSeparator()
@@ -120,14 +125,12 @@ class RangeTreeWidget(QTreeWidget):
 				if isinstance(source.fi, SwitchFI):
 					ctx.addAction("Add case ...", lambda: self.addField(source, "SwitchItem"))
 					ctx.addSeparator()
-				if parentSource is not None and isinstance(parentSource.fi, (StructFI, UnionFI, BitStructFI)):
-					ctx.addAction("Remove this field", lambda: self.removeField(parentSource, range.field_name))
-					ctx.addSeparator()
-				ctx.addAction("Edit ...", lambda: self.editField(source))
-				ctx.addAction("Edit tree ...", lambda: self.editField2(source))
 				ctx.addAction("Visualization ...", lambda: self.editDisplayParams(source))
 				ctx.addAction("Repeat ...", lambda: self.repeatField(source))
-				ctx.addAction("Hide", lambda: self.styleSelection(source, hide=1))
+				ctx.addAction("Hide field", lambda: self.styleSelection(source, hide=1))
+				if parentSource is not None and isinstance(parentSource.fi, (StructFI, UnionFI, BitStructFI)):
+					ctx.addAction("Delete field", lambda: self.removeField(parentSource, item_range.field_name))
+					ctx.addSeparator()
 				ctx.addSeparator()
 				for key, name, style in guihelper.getHighlightStyles():
 					ctx.addAction(name+"\t"+key, lambda style=style: self.styleSelection(source, **style))
@@ -161,20 +164,24 @@ class RangeTreeWidget(QTreeWidget):
 
 	def removeField(self, parent, field_name):
 		ch = parent.params['children']
-		del ch[next(i for i,(key,el) in enumerate(ch) if key == field_name)]
+		del ch[next(i for i,(comment,key,el) in enumerate(ch) if key == field_name)]
 		parent.updateParams(children=ch)
 		self._afterUpdate()
 
 	def editDisplayParams(self, parent):
+		from pre_workbench.structinfo import ExprFunctions
+		showModes = [("","")] + ExprFunctions.getSelectList('name')
 		params = showSettingsDlg([
 			SettingsField("color", "Background color", "color", {"color":True}),
 			SettingsField("textcolor", "Text color", "color", {"color":True}),
 			SettingsField("section", "Section header", "text", {}),
+			SettingsField("show", "Value display style", "select", {"editable":True,"options":showModes}),
 		], title="Edit display params", values=parent.params, parent=self)
 		if params is None: return
 		if params.get("color") == "": params["color"] = None
 		if params.get("textcolor") == "": params["textcolor"] = None
 		if params.get("section") == "": params["section"] = None
+		if params.get("show") == "": params["show"] = None
 		parent.updateParams(**params)
 		self._afterUpdate()
 
@@ -193,7 +200,7 @@ class RangeTreeWidget(QTreeWidget):
 		def ok_callback(result):
 			element.from_text(result)
 			self._afterUpdate()
-		showScintillaDialog(self, "Edit field", element.to_text(0, None), ok_callback=ok_callback)
+		showScintillaDialog(self, "Edit field", element.to_text(0, None), ok_callback=ok_callback, help_callback=lambda: navigateBrowser(SYNTAX_REFERENCE_URL))
 
 	def editField2(self, element: FormatInfo):
 		def ok_callback(params):
