@@ -21,6 +21,7 @@ import re
 import subprocess
 import tempfile
 import weakref
+from binascii import hexlify
 from copy import deepcopy
 
 import yaml
@@ -35,7 +36,7 @@ import pre_workbench.app
 from pre_workbench import configs
 from pre_workbench.algo.range import Range
 from pre_workbench.configs import getIcon, SettingsField
-from pre_workbench.consts import MACRO_CODE_HELP_URL, MACRO_PROPERTIES_HELP_URL
+from pre_workbench.consts import MACRO_CODE_HELP_URL, MACRO_PROPERTIES_HELP_URL, SYNTAX_REFERENCE_URL
 from pre_workbench.controls.genericwidgets import showSettingsDlg
 from pre_workbench.controls.scintillaedit import showScintillaDialog
 from pre_workbench.errorhandler import ConsoleWindowLogHandler
@@ -94,19 +95,22 @@ class FileBrowserWidget(QWidget):
 			selectedFile = file.absoluteFilePath()
 			selectedFolder = selectedFile if file.isDir() else file.absolutePath()
 			if file.isDir():
-				ctx.addAction("Open as directory of binary files", lambda: navigate("WINDOW", "Type=ObjectWindow", "FileName=" + selectedFile,
+				ctx.addAction("Open as Directory of Binary Files", lambda: navigate("WINDOW", "Type=ObjectWindow", "FileName=" + selectedFile,
 																			   "dataSourceType=DirectoryOfBinFilesDataSource"))
 				ctx.addSeparator()
-				ctx.addAction("Open in file manager", lambda: QDesktopServices.openUrl(QUrl.fromLocalFile(selectedFile)))
+				ctx.addAction("Open in File Manager", lambda: QDesktopServices.openUrl(QUrl.fromLocalFile(selectedFile)))
 			else:
+				ctx.addAction("Open as Data Source", lambda: navigate("WINDOW", "Type=ObjectWindow", "FileName=" + selectedFile,
+																			   "dataSourceType=FileDataSource"))
+				mnuOpenWith = ctx.addMenu('Open with ...')
 				for wndTyp, meta in WindowTypes.types:
-					text = 'Open with '+meta.get('displayName', meta['name'])
-					ctx.addAction(QAction(text, self, statusTip=text, icon=getIcon(meta.get('icon', 'document.png')),
+					text = meta.get('displayName', meta['name'])
+					mnuOpenWith.addAction(QAction(text, self, statusTip=text, icon=getIcon(meta.get('icon', 'document.png')),
 											   triggered=lambda dummy, meta=meta: navigate("WINDOW", "Type="+meta['name'], "FileName="+selectedFile)))
 				ctx.addSeparator()
-				ctx.addAction("Open in default application", lambda: QDesktopServices.openUrl(QUrl.fromLocalFile(selectedFile)))
+				ctx.addAction("Open in Default Application", lambda: QDesktopServices.openUrl(QUrl.fromLocalFile(selectedFile)))
 		else:
-			ctx.addAction("Open in file manager", lambda: QDesktopServices.openUrl(QUrl.fromLocalFile(self.model.rootPath())))
+			ctx.addAction("Open in File Manager", lambda: QDesktopServices.openUrl(QUrl.fromLocalFile(self.model.rootPath())))
 		#ctx.addAction("Set root folder ...", lambda: self.selectRootFolder(preselect=selectedFolder))
 		ctx.exec(self.tree.viewport().mapToGlobal(point))
 
@@ -150,9 +154,9 @@ class MdiWindowListWidget(QWidget):
 
 	def _initUI(self):
 		self.list = QListWidget()
-		#self.list.doubleClicked.connect(self.onDblClick)
-		self.list.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
-		self.list.customContextMenuRequested.connect(self.onCustomContextMenuRequested)
+		# TODO fix context menu
+		#self.list.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+		#self.list.customContextMenuRequested.connect(self.onCustomContextMenuRequested)
 		self.list.itemClicked.connect(self.gotoItem)
 
 		windowLayout = QVBoxLayout()
@@ -217,21 +221,31 @@ class StructInfoCodeWidget(QWidget):
 	def __init__(self):
 		super().__init__()
 		self._initUI()
+		self.editor.ctrlEnterPressed.connect(self._applyContent)
+		self.editor.modificationChanged.connect(self._modificationChanged)
 		self._updateContent()
 		pre_workbench.app.CurrentProject.formatInfoContainer.updated.connect(self._updateContent)
-		self.editor.ctrlEnterPressed.connect(self._applyContent)
 
 	def _updateContent(self):
 		self.editor.setText(pre_workbench.app.CurrentProject.formatInfoContainer.to_text())
+		self.editor.setModified(False)
 
 	def _applyContent(self):
 		pre_workbench.app.CurrentProject.formatInfoContainer.load_from_string(self.editor.text())
 		pre_workbench.app.CurrentProject.formatInfoContainer.write_file(None)
+		self.editor.setModified(False)
+
+	def _modificationChanged(self, mod):
+		self.toolbar.setVisible(mod)
 
 	def _initUI(self):
+		self.toolbar = QToolBar()
+		self.toolbar.addAction("Apply Definitions", self._applyContent)
+		self.toolbar.addAction("Syntax Reference", lambda: navigateBrowser(SYNTAX_REFERENCE_URL))
 		self.editor = ScintillaEdit()
 		windowLayout = QVBoxLayout()
 		windowLayout.addWidget(self.editor)
+		windowLayout.addWidget(self.toolbar)
 		windowLayout.setContentsMargins(0,0,0,0)
 		self.setLayout(windowLayout)
 		self.editor.show()
@@ -392,6 +406,15 @@ class MacroListDockWidget(QWidget):
 	option_types=["-","text","color","font","select","check","int"]
 	CONTAINER_ROLE = QtCore.Qt.UserRole + 100
 	MACRO_NAME_ROLE = QtCore.Qt.UserRole + 101
+	MACRO_ICONS = {
+		'NONE': 'script.png',
+		'BYTE_ARRAY': 'script-attribute.png',
+		'BYTE_BUFFER': 'script-attribute-b.png',
+		'BYTE_BUFFER_LIST': 'script-attribute-l.png',
+		'DATA_SOURCE': 'script-attribute-d.png',
+		'STRING': 'script-attribute-s.png',
+		'SELECTION_HEURISTIC': 'script-attribute-h.png',
+	}
 	def __init__(self):
 		super().__init__()
 		self._initUI()
@@ -415,15 +438,17 @@ class MacroListDockWidget(QWidget):
 		for container_id, container in APP().macro_containers.items():
 			root = QTreeWidgetItem(self.treeView, [container.containerTitle])
 			root.setExpanded(True)
+			root.setIcon(0, getIcon("box.png"))
 			root.setData(0, MacroListDockWidget.CONTAINER_ROLE, container)
 			self._loadMacros(root, container)
 
 	def _loadMacros(self, root, container):
-		names = container.getMacroNames()
-		for name in names:
-			item = QTreeWidgetItem(root, [name])
+		macroList = container.getMacros()
+		for macroItem in macroList:
+			item = QTreeWidgetItem(root, [macroItem.name])
+			item.setIcon(0, getIcon(MacroListDockWidget.MACRO_ICONS.get(macroItem.input_type, "script.png")))
 			item.setData(0, MacroListDockWidget.CONTAINER_ROLE, container)
-			item.setData(0, MacroListDockWidget.MACRO_NAME_ROLE, name)
+			item.setData(0, MacroListDockWidget.MACRO_NAME_ROLE, macroItem.name)
 
 	def _customContextMenuRequested(self, point):
 		item = self.treeView.itemAt(point)
@@ -459,13 +484,14 @@ class MacroListDockWidget(QWidget):
 	def _onDblClick(self, item: QTreeWidgetItem, columnIdx: int):
 		container = item.data(0, MacroListDockWidget.CONTAINER_ROLE)
 		macroname = item.data(0, MacroListDockWidget.MACRO_NAME_ROLE)
-		self.executeMacro(container.getMacro(macroname))
+		if macroname:
+			self.executeMacro(container.getMacro(macroname))
 
 	def executeMacro(self, macro):
 		if macro.input_type == Macro.TYPE_NONE:
 			macro.execute(None)
 		else:
-			TODO()
+			QMessageBox.warning(self, "Not implemented", "Please call with input data of type: " + macro.input_type)
 
 	def exportMacro(self, macro):
 		fileName, _ = QFileDialog.getSaveFileName(self, "Export Macro", os.path.join(APP().project.projectFolder, macro.name + ".macro.yml"), "Macro (*.macro.yml)")
@@ -724,17 +750,22 @@ class SearchDockWidget(QWidget):
 
 	def _initUI(self):
 		toolbar = QToolBar()
-		toolbar.addAction("Find", self._runTool)
 		self.cmdLineEdit = QComboBox(editable=True, minimumWidth=200)
 		QShortcut(QKeySequence(QtCore.Qt.Key_Return), self.cmdLineEdit, activated=self._runTool, context=QtCore.Qt.WidgetWithChildrenShortcut)
 		self._updateMru()
 		toolbar.addWidget(self.cmdLineEdit)
+		toolbar.addAction(getIcon("magnifier-left.png"), "Find", self._runTool)
+		self.displayAsHexAction = QAction(getIcon("document-binary.png"), "Display Results as Hex String", self, triggered=self._runTool)
+		self.displayAsHexAction.setCheckable(True)
+		toolbar.addAction(self.displayAsHexAction)
+
 		self.treeView = QTreeWidget()
 		self.treeView.setColumnCount(2)
 		self.treeView.setColumnWidth(0, 300)
 		self.treeView.headerItem().setText(0, "Result")
 		self.treeView.headerItem().setText(1, "Offset")
-		self.treeView.itemClicked.connect(self._itemClicked)
+		self.treeView.currentItemChanged.connect(self._currentItemChanged)
+		self.treeView.setSelectionMode(QAbstractItemView.ExtendedSelection)
 		windowLayout = QVBoxLayout()
 		windowLayout.addWidget(toolbar)
 		windowLayout.addWidget(self.treeView)
@@ -764,14 +795,18 @@ class SearchDockWidget(QWidget):
 					root.setText(0, "Buffer %d: %r" % (i, buf.metadata))
 					root.setExpanded(True)
 				x = QTreeWidgetItem(root)
-				x.setText(0, match.group(0).decode('latin1'))
+				if self.displayAsHexAction.isChecked():
+					x.setText(0, hexlify(match.group(0), b':').decode('latin1'))
+				else:
+					x.setText(0, match.group(0).decode('latin1'))
 				x.setText(1, str(match.start()))
 				x.setData(0, QtCore.Qt.UserRole, Range(match.start(), match.end(), buffer_idx=i))
 
-	def _itemClicked(self, item):
+	def _currentItemChanged(self, item, prevItem):
+		if not item: return
 		range = item.data(0, QtCore.Qt.UserRole)
-		if range:
-			self.hexview.selectRange(range, True)
+		if not range: return
+		self.hexview.selectRange(range, True)
 
 
 
